// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  CatalogDomainInfo.java
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// wi8754       WXS 8.5   03/23/2012   cdjohnson new part
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.util.Collection;

/**
 * Provides access to catalog domain configuration information for the current environment.<p>
 * 
 * When running in a WebSphere Application Server profile augmented with WebSphere eXtreme Scale, the 
 * CatalogDomainManager returns the catalog service domain configuration information that is configured
 * in the administration console. 
 * 
 * @see ObjectGridManager#getCatalogDomainManager()
 * @since 8.5, XC10
 */
public interface CatalogDomainManager {

    /**
     * Retrieve the default, configured CatalogDomainInfo.
     * @return the default CatalogDomainInfo, or null if not available.
     */
    public CatalogDomainInfo getDefaultDomainInfo(); 
    
    /**
     * Retrieve the specified CatalogDomainInfo for the specified domainId.
     * @param domainId the domain identifier.
     * @return the CatalogDomainInfo if found, or null.
     */
    public CatalogDomainInfo getDomainInfo(String domainId);

    /**
     * Retrieve all configured CatalogDomainInfo objects.
     * @return a collection of CatalogDomainInfo
     */
    public Collection<CatalogDomainInfo> getDomainInfos();
}
