// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/ConnectException.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:20:36
//
// Module  :  ConnectException.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   10/26/2005   rajeshk    new code drop
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid;

/**
 * This exception is used to indicate that the client was unable to connect to the server
 *
 * @since WAS XD 6.0.1, XC10
 */
public class ConnectException extends ObjectGridException
{
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = -4497037898915673731L;

    /**
     * This failure code indicates the provided configuration was corrupt.
     */
    public static final int BAD_CONFIGURATION = 1;

    /**
     * This failure code indicates the reason for the connect failure is unknown.
     */
    public static final int UNKNOWN = 2;

    /**
     * This failure code indicates the a failure to authenticate.
     */
    public static final int FAILED_SECURITY = 3;

    /**
     * This failure code indicates that the server may not be available.
     */
    public static final int CONNECTION_REFUSED = 4;

    /**
     * This failure code indicates the definition of cluster cannot be accessed.
     */
    public static final int SERVER_DEFINITION_NOT_FOUND = 5;

    private int failureCode;

    /**
     * Constructs a new ConnectException with <code>null</code> as its detail
     * message. The cause is not initialized, and may subsequently be initialized
     * by a call to the <code>initCause</code> method.  In addition the failure
     * code is initialized to <code>UNKNOWN</code>.
     *
     * @see #initCause(Throwable)
     * @see #UNKNOWN
     */
    public ConnectException() {
        super();
        failureCode=UNKNOWN;
    }

    /**
     * Constructs a new ConnectException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.   In addition the failure
     * code is initialized to <code>UNKNOWN</code>.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     * @see #UNKNOWN
     */
    public ConnectException(String message) {
        super(message);
        failureCode=UNKNOWN;
    }

    /**
     * Constructs a new ConnectException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * @param failureCode the failure code which should be one of the
     *        constants of this exception class.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     * @see #getFailureCode
     */
    public ConnectException(String message, int failureCode) {
        super(message+" <"+failureCode+"> ");
        this.failureCode=failureCode;
    }

    /**
     * Constructs a new ConnectException with a specified cause.
     * The cause and a detail message of <code>(cause==null ? null : cause.toString())</code>
     * is used (which typically contains the class and detail message of <code>cause</code>).
     * This constructor is useful for ConnectExceptions that are little more
     * than wrappers for other throwables. The failure code is initialized to
     * <code>UNKNOWN</code>.
     *
     * @param cause is the exception that caused this exception to be thrown,
     *        which is saved for later retrieval by the <code>getCause()</code> method.
     *        A <code>null</code> value is permitted and indicates that the cause
     *        is nonexistent or is unknown.
     *
     * @see #getCause()
     * @see #UNKNOWN
     */
    public ConnectException(Throwable cause) {
        super(cause);
        failureCode=UNKNOWN;
    }

    /**
     * Constructs a new ConnectException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ConnectException's detail
     * message.    The failure code is initialized to <code>UNKNOWN</code>.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     *
     * @see #getCause()
     * @see #getMessage()
     */
    public ConnectException(String message, Throwable cause) {
        super(message, cause);
        failureCode=UNKNOWN;
    }

    /**
     * Constructs a new ConnectException with the specified detail message and
     * cause.
     * <p>
     * Note that the detail message associated with <code>cause</code> is
     * <i>not</i> automatically incorporated in this ConnectException's detail
     * message.    The failure code is initialized to <code>UNKNOWN</code>.
     *
     * @param message
     *           the detail message (which is saved for later retrieval by the
     *           <code>getMessage</code> method).
     * @param cause
     *           the cause (which is saved for later retrieval by the
     *           <code>getCause</code> method). (A<code>null</code> value is
     *           permitted, and indicates that the cause is nonexistent or
     *           unknown).
     *
     * @param failureCode the failure code which should be one of the
     *        constants of this exception class.
     *
     * @see #getCause()
     * @see #getMessage()
     * @see #getFailureCode()
     */
    public ConnectException(String message, Throwable cause, int failureCode) {
        super(message+" <"+failureCode+"> ", cause);
        this.failureCode=failureCode;
    }

    /**
     * Returns the failure code that was set by one of the constructors that
     * accepts a failure code, or <code>UNKNOWN</code> if one of the other
     * constructors was called.
     *
     * @return the failure code.  One of the constants of this exception class.
     */
    public int getFailureCode() {
        return failureCode;
    }
}
