// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.2 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/TransactionTimeoutException.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:25:45
//
// Module  : TransactionTimeoutException.java
//
// Source File Description:
//
//    See javadoc for class.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// DOW          WASX.XD   Nov 17, 2005 kjlaw     add transaction timeout feature.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid;

import java.util.Date;

/**
 * This exception is thrown when a transaction exceeds the transaction timeout
 * that was specified on the <code>ObjectGrid</code> or <code>Session</code>.
 *
 * @since WAS XD 6.0.1, XC10
 *
 * @see ObjectGrid#setTxTimeout(int)
 * @see Session#setTransactionTimeout(int)
 */
public class TransactionTimeoutException extends ObjectGridRuntimeException
{
    /**
     * The <code>serialVersionUID</code> for this class.
     */
    private static final long serialVersionUID = 4673938838136171299L;

    /**
     * The instant in time when this exception was created.
     */
    final private Date ivOccuranceTime = new Date();

    /**
     * String representation of the TxID for the transaction that timed out.
     */
    private String ivTxIdString;
    
    public TransactionTimeoutException(String message, Throwable cause) {
    	super(message, cause);
    	ivTxIdString = "";
    }

    /**
     * Constructs a new TransactionTimeoutException with the specified detail message. The
     * cause is not initialized, and may subsequently be initialized by a call to
     * the <code>initCause</code> method.
     *
     * @param message
     *           the detail message. The detail message is saved for later
     *           retrieval by the <code>getMessage</code> method.
     * @param txIdString
     *           the result of <code>TxID.toString()</code> for the transaction that timed out.
     *
     * @see #initCause(Throwable)
     * @see #getMessage()
     */
    public TransactionTimeoutException(String message, String txIdString)
    {
        super(message);
        ivTxIdString = txIdString;
    }

    /**
     * Gives the time when this TransactionTimeoutException was created.
     *
     * @return Date object that represents the instant in time when this
     *         exception object was created.
     */
    public Date whenOccurred()
    {
        return ivOccuranceTime;
    }

    /**
     * Get the String representation of the <code>TxID</code> for the
     * transaction that timed out.
     *
     * @return  String value of TxID of transaction that timed out.
     */
    public String getTxIDString()
    {
        return ivTxIdString;
    }
}