/*
 * IBM Confidential OCO Source Material
 * 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008, 2013
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 *
 * $HeadURL$
 * $Id$
 *
 * Change Activity:
 *
 * Reason       Version   Date         Userid    Change Description
 * ------------ --------- ------------ --------- ---------------------------------------
 *               6.1.0.3  04/06/2008   haowang   created
 * b16176       7.0       06/02/2008   jian.tang deprecate com.ibm.websphere.objectgrid.ClientProperties
 * b18390       XS7.1     03/26/2010   jaredp    Added getListenerPort and getListenerHost
 * b19712       XS7.1     05/04/2010   jaredp    Corrected javadoc for listener host and port code
 * b21833       XS7.1.x   11/10/2010   jian.tang preferLocalHost and preferLocalProcess are not used.
 * b22487       6105 FIX5 03/14/2011   westland  Add support to use new ReadOnlyCatalogService
 * b22852       7.1.1     05/05/2011   westland  Update javadoc for BootStraplistShuffled property
 */

package com.ibm.websphere.objectgrid.client;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.server.ServerProperties;

/**
 * The set of properties used to define various preference for ObjectGrid clients.
 * <p>
 * 
 * See the <a href="package-summary.html">package summary</a> for details on how to use the ClientProperties class and properties file.
 * 
 * @since WAS XD 6.1.0.3, XC10
 */
public interface ClientProperties {

    /**
     * The default name of client property file
     */
    public static final String DEFAULTCLIENTPROPERTYFILE = "objectGridClient.properties";

    /**
     * The system property key to override the location of the client properties file.
     * 
     * @since 7.0
     */
    public final static String CLIENT_PROPS_FILE_PATH_KEY = "objectgrid.client.props";

    /**
     * Currently, this property is not used. It is reserved for future use.
     * 
     * @see #setPreferLocalProcess(boolean)
     */
    public static final String PROP_PREFER_LOCAL_PROCESS = "preferLocalProcess";

    /**
     * Currently, this property is not used. It is reserved for future use.
     * 
     * @see #setPreferLocalHost(boolean)
     */
    public static final String PROP_PREFER_LOCAL_HOST = "preferLocalhost";

    /**
     * Prefer zones property key for the client properties file. Each specified zone is separated by a comma in the form:
     * preferZones=ZoneA,ZoneB,ZoneC
     * 
     * @see #setPreferZones(String[])
     */
    public static final String PROP_PREFER_ZONES = "preferZones";

    /**
     * The requestRetryTimeout which indicates how long to retry a request (in milliseconds). A 0 indicates that the request should fail fast and skip
     * over in internal retry logic. Exceptions that cannot succeed even if tried again such as DuplicateException will be returned immediately.
     * 
     * @see #setRequestRetryTimeout(long)
     * @since 7.0
     */
    public static final String PROP_REQUEST_RETRY_TIMEOUT = "requestRetryTimeout";

    /**
     * Listener host property key for the client properties file.
     * 
     * @see #getListenerHost()
     * @since XS 7.1
     */
    public static final String PROP_LISTENER_HOST = "listenerHost";

    /**
     * Listener port property key for the client properties file.
     * 
     * @see #getListenerPort()
     * @since XS 7.1
     */
    public static final String PROP_LISTENER_PORT = "listenerPort";

    /**
     * The shuffleBoostrapAddresses property is used to determine if the catalog service grid addresses should be randomized when used by a client
     * when bootstrapping to the grid.  The default value of the property is true. 
     * 
     * @since 7.1.0.3
     */
    public static final String PROP_SHUFFLE_BOOTSTRAP_ADDRESSES = "shuffleBootstrapAddresses";

    /**
     * Sets the timeout for server requests using the eXtremeIO transport. The timeout is set in seconds.
     * The default xioTimeout for server requests is set to 30 seconds. The valid range is timeout >= 1.
     *
     * @since 8.6
     */
    public static final String PROP_XIO_TIMEOUT =ServerProperties.PROP_XIO_TIMEOUT;
    
    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @since 8.6
     */    
    public static final String PROP_MAXIMUM_XIO_NETWORK_THREAD_POOL_SIZE =ServerProperties.PROP_MAXIMUM_XIO_NETWORK_THREAD_POOL_SIZE;
    
    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @since 8.6
     */
    public static final String PROP_MAXIMUM_XIO_WORKER_THREAD_POOL_SIZE =ServerProperties.PROP_MAXIMUM_XIO_WORKER_THREAD_POOL_SIZE;
    
    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @since 8.6
     */
    public static final String PROP_MINIMUM_XIO_NETWORK_THREAD_POOL_SIZE =ServerProperties.PROP_MINIMUM_XIO_NETWORK_THREAD_POOL_SIZE;
    
    
    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @since 8.6
     */
    public static final String PROP_MINIMUM_XIO_WORKER_THREAD_POOL_SIZE =ServerProperties.PROP_MINIMUM_XIO_WORKER_THREAD_POOL_SIZE;
    
    /**
     * Prefer routing to specific zones.
     * <p>
     * 
     * When zones are enabled on an ObjectGrid, requests will be routed to the specified zones.
     * <p>
     * 
     * @param zones
     *            array of zone names. If null or an empty array, then requests are routed to all zones.
     */
    public void setPreferZones(String[] zones);

    /**
     * This method is reserved for future use. Calls to the method will not result in any performed operation.
     * 
     * @param localProcess
     */
    void setPreferLocalProcess(boolean localProcess);

    /**
     * This method is reserved for future use. Calls to the method will not result in any performed operation.
     * 
     * @param localHost
     */
    void setPreferLocalHost(boolean localHost);

    // FUTURE
    // /**
    // * Enables or disables client to server to client message compression. Messages will be compressed by default.
    // *
    // * @param enabled
    // * Whether or not messages to and from the server should be compressed.
    // */
    // public void enableCompression(boolean enabled);

    /**
     * Retrieve the preferred zones.
     * 
     * @return the preferred zones.
     */
    public String[] getPreferZones();

    /**
     * This method is reserved for future use. The returned value should be ignored by the user.
     * 
     * @return false
     */
    public boolean isPreferLocalProcess();

    /**
     * This method is reserved for future use. The returned value should be ignored by the user.
     * 
     * @return false
     */
    public boolean isPreferLocalHost();

    // FUTURE
    // public boolean isCompressionEnabled();

    /**
     * Set the request retry timeout to indicate how long to retry a request (in milliseconds) when recoverable failures occur, such as fail-over
     * exceptions. A request will timeout when either the request timeout expires or the transaction timeout expires, whichever expires first.
     * <p>
     * 
     * A value of 0 indicates that all requests should fail immediately and avoid any retry logic. Exceptions that cannot succeed even if tried again
     * such as DuplicateKeyException exceptions will be thrown immediately.
     * <p>
     * 
     * A value of -1 indicates that the request retry timeout is not set, meaning that the request duration is governed by the transaction timeout.
     * <p>
     * 
     * The request retry timeout can be overridden using the {@link Session#setRequestRetryTimeout(long)} method.
     * 
     * @param requestRetryTimeout
     *            the duration in milliseconds retry a client request, 0 if the request should fail immediately or -1 if the request timeout is not
     *            set.
     * @see Session#setRequestRetryTimeout(long)
     * @see ObjectGrid#setTxTimeout(int)
     * @since 7.0
     */
    public void setRequestRetryTimeout(long requestRetryTimeout);

    /**
     * Retrieves the current request retry timeout. Returns -1 if it was not set.
     * 
     * @return requestRetryTimeout in milliseconds, 0 to fail immediately or -1 if not set.
     * @since 7.0
     */
    public long getRequestRetryTimeout();

    /**
     * Retrieves the host to be used by the ORB. The listener host property defaults to 'localhost'. This property can only be set in the
     * client.properties file.
     * 
     * @return The host that the ORB will bind to.
     * @since 7.1
     */
    public String getListenerHost();

    /**
     * Retrieves the port to be used by the ORB. The listener port property defaults to the corbaloc port, 2809. This property can only be set in the
     * client.properties file.
     * 
     * @return The port that the ORB will bind to.
     * @since 7.1
     */
    public int getListenerPort();

    /**
     * Retrieves the value of the BootStrapListShuffled property.
     * 
     * @return true if the value of BootStrapListeShuffled was set to true. 
     *         false if the value of BootStrapListeShuffled was set to false.
     * @since 7.1.0.3
     * 
     */

    public boolean isBootStrapListShuffled();

    /**
     * Sets the value of the BootStrapListShuffled property.
     * 
     * @param shuffle
     *            true the bootstrap list will be shuffled providing each client a random distribution of 
     *                 catalog servers to select from.
     *            false the first viable address in the list of catalog servers will be used. 
     * @since 7.1.0.3
     */
    public void setBootStrapListShuffled(boolean shuffle);
    
    /**
     * Retrieves the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @return the mininum number of threads.
     * @since 8.6
     */
    public int getMinimumXIOWorkerThreads();

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @param minThreads the minimum number of threads.
     * @since 8.6
     */
    public void setMinimumXIOWorkerThreads(int minThreads);

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @return the maximum number of threads.
     * @since 8.6
     */
    public int getMaximumXIOWorkerThreads();

    /**
     * Retrieves the maximum number of threads to allocate in the eXtremeIO transport request
     * processing thread pool.
     * @param maxThreads the maximum number of threads.
     * @since 8.6
     */
    public void setMaximumXIOWorkerThreads(int maxThreads);

    /**
     * Gets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @return the minimum number of threads
     * @since 8.6
     */
    public int getMinimumXIONetworkThreads();

    /**
     * Sets the minimum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @param minThreads the minimum number of threads
     * @since 8.6
     */
    public void setMinimumXIONetworkThreads(int minThreads);

    /**
     * Gets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @return the maximum number of threads.
     * @since 8.6
     */
    public int getMaximumXIONetworkThreads() ;

    /**
     * Sets the maximum number of threads to allocate in the eXtremeIO transport network
     * thread pool.
     * @param maxThreads the maximum number of threads.
     * @since 8.6
     */
    public void setMaximumXIONetworkThreads(int maxThreads);
    
    /**
     * Returns the current timeout for server requests using the XIO transport.
     *
     * @return the current timeout in seconds
     * @since 8.6
     */
    public int getXIOTimeout();

    /**
     * Sets the current timeout for server requests using the XIO transport. The timeout is
     * set in seconds. The valid range is timeout >= 1.
     *
     * @param timeout the timeout in seconds
     * @since 8.6
     */
    public void setXIOTimeout(int timeout);


}
