// IBM Confidential OCO Source Material
// 5724-J34 COPYRIGHT International Business Machines Corp. 2005-2011
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  ObjectGridConfiguration.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid    Change Description
// --------- --------- ------------ --------- -----------------------------------------
// b896                05/01/2006   vandersn  open config APIs
// b23304    7.1.0.3   04/24/2011   jhanders  Add get/setTxTimeout
// --------- --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.config;

import java.util.List;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;

/**
 * An ObjectGridConfiguration object can be used to override ObjectGrid plugins on the
 * client side.  The <code>com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener</code>
 * and the <code>com.ibm.websphere.objectgrid.plugins.TransactionCallback</code> Plugins can be overridden.
 *
 * @see com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener
 * @see com.ibm.websphere.objectgrid.plugins.TransactionCallback
 * @since WAS XD 6.0.1.2, XC10
 */

public interface ObjectGridConfiguration {


    /**
     * Get the name of this ObjectGridConfiguration
     * @return the name of this ObjectGridConfiguration
     */
    public String getName();

    /**
     * Add a BackingMapConfiguration to this ObjectGridConfiguration.
     * @param backingMapConfiguration
     */
	public void addBackingMapConfiguration(BackingMapConfiguration backingMapConfiguration);

    /**
     * Set the BackingMapConfiguration objects for this ObjectGridConfiguration.  Any BackingMapConfiguration
     * objects that were previously attached to this ObjectGridConfiguration object will be overridden.
     * @param backingMapConfigList - A List of <code>BackingMapConfiguration</code> objects.
     * @see BackingMapConfiguration
     */
	public void setBackingMapConfigurations(List backingMapConfigList);

    /**
     * Get the List of BackingMapConfiguration objects that are attached to this ObjectGridConfiguration object
     * @return a List of <code>BackingMapConfiguration</code> objects
     * @see BackingMapConfiguration
     */
	public List getBackingMapConfigurations();

    /**
     * Add a <code>Plugin</code> to this ObjectGridConfiguration.  The Plugins that can be overridden on a
     * client-side ObjectGrid are <code>com.ibm.websphere.objectgrid.plugins.ObjectGridEventListener</code>
     * and <code>com.ibm.websphere.objectgrid.plugins.TransactionCallback</code>.
     * @param plugin
     * @see #setPlugins(List)
     * @see Plugin
     */
	public void addPlugin(Plugin plugin);

    /**
     * Set the Plugins for this ObjectGridConfiguration.  Any Plugins that were previously attached to this
     * ObjectGridConfiguration object will be overridden.
     * @param pluginList - a List of Plugins
     * @see #addPlugin(Plugin)
     * @see Plugin
     */
	public void setPlugins(List pluginList);

    /**
     * Get the Plugins that have been attached to this ObjectGridConfiguration.
     * @return a List of <code>Plugin</code> objects
     * @see Plugin
     */
	public List getPlugins();

	/**
	 * Sets the transaction timeout
	 *
	 * @param seconds
	 * @see ObjectGrid#setTxTimeout(int)
	 * @since 7.1.0.3
	 */
    public void setTxTimeout(int seconds);

    /**
     * Gets the transaction timeout.  The value is in seconds.
     *
     * @return the transaction timeout in seconds
     * @see ObjectGrid#getTxTimeout()
     * @since 7.1.0.3
     */
    public int getTxTimeout();

    /**
     * Sets the default transaction isolation level for all sessions created by the ObjectGrid.
     * The constants defined in the <tt>Session</tt>
     * interface are the possible transaction isolation levels.
     * The default is {@link Session#TRANSACTION_REPEATABLE_READ}.
     *
     * @param level one of the following <tt>Session</tt> constants:
     * {@link Session#TRANSACTION_READ_UNCOMMITTED}, {@link Session#TRANSACTION_READ_COMMITTED} or
     * {@link Session#TRANSACTION_REPEATABLE_READ} or 0 if the TransactionIsolation should not be set.
     * @since 7.1.1
     */
    void setTxIsolation(int level);

    /**
     * Retrieves the default transaction isolation level.
     * @return the current transaction isolation level.
     * @see #setTxIsolation(int)
     * @since 7.1.1
     */
    int getTxIsolation();
}
