// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  AgentManagerMBean.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code
// ------------ --------- ------------ --------- -----------------------------------------


package com.ibm.websphere.objectgrid.management;

import com.ibm.websphere.objectgrid.stats.AgentStatsModule;


/**
 * This MBean interface allows a client process to access different attributes
 * and statistical data about a specific Agent on a server process.  The Agent Manager
 * MBean is scoped at the map level and therefore can access statistical data for every agent run against
 * the specified map.  In a dynamic ObjectGrid environment, the object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=AgentManager,name=Agent-&lt;map&gt;,partition=&lt;partition id&gt;,objectgrid=&lt;objectgrid&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.1.0.5, XC10
 */

public interface AgentManagerMBean {

	/**
     * Gets the specified agent's total reduce time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the reduce time for this agent in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getReduceTime(boolean copy)
     */

	double getReduceTime(String agentClassName);

	/**
     * Gets the specified agent's total run time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the total run time for this agent in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getTotalDurationTime(boolean copy)
     */

	double getTotalDurationTime(String agentClassName);

	/**
     * Gets the specified agent's serialization time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the time it takes to serialize the agent in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getAgentSerializationTime(boolean copy)
     */

	double getAgentSerializationTime(String agentClassName);

	/**
     * Gets the specified agent's inflation time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the time it takes to inflate the agent in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getAgentInflationTime(boolean copy)
     */

	double getAgentInflationTime(String agentClassName);

	/**
     * Gets the specified agent's result inflation time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the time it takes to inflate the agent results for a given partition in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getResultInflationTime(boolean copy)
     */

	double getResultInflationTime(String agentClassName);

	/**
     * Gets the specified agent's result serialization time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the time it takes to serialize the agent results for a given partition in milliseconds
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getResultSerializationTime(boolean copy)
     */

	double getResultSerializationTime(String agentClassName);

	/**
     * Gets the specified agent's partition count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the number of partitions this agent is sent to
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getPartitionCount(boolean copy)
     */

	double getPartitionCount(String agentClassName);

	/**
     * Gets the specified agent's failure count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the failure count for the specified agent
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getFailureCount(boolean copy)
     */

	double getFailureCount(String agentClassName);

	/**
     * Gets the specified agent's invocation count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return the invocation count for the specified agent
     *
     * @see #retrieveStatsModule(String)
     * @see AgentStatsModule#getInvocationCount(boolean copy)
     */

	double getInvocationCount(String agentClassName);

	/**
     * Gets the <code>AgentStatsModule</code> used to retrieve statistics
     * associated with the specified agent class
     *
     * @param agentClassName The fully qualified class name of the agent
     * @return an AgentStatsModule for statistics associated with the specified agent class
     *
     * @see AgentStatsModule
     */

	public AgentStatsModule retrieveStatsModule(String agentClassName);

}
