// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2008
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// Module  :  MapIndexMBean.java
//
// Source File Description:
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// 13570         XD6105    12-4-2008   bsandman  code drop
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.management;

import com.ibm.websphere.objectgrid.TxID;
import com.ibm.websphere.objectgrid.plugins.LogSequence;
import com.ibm.websphere.objectgrid.plugins.index.MapIndexPlugin;
import com.ibm.websphere.objectgrid.stats.HashIndexStatsModule;

/**
 * This MBean interface allows a client process to access different attributes
 * and statistical data about a specific HashIndex on a server process.  In a dynamic ObjectGrid environment, the object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=HashIndex,name=&lt;index-name&gt;,partition=&lt;partition id&gt;,objectgrid=&lt;objectgrid&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.1.0.5, XC10
 */

public interface HashIndexMBean {

	/**
	 * Gets the index's parent map name
	 *
	 * @return the name of the map which this index belongs to
	 */

	String getParentMapName();

	/**
     * Gets the <code>HashIndexStatsModule</code> used to retrieve statistics
     * associated with this index
     *
     * @return an HashIndexStatsModule for statistics associated with this index
     *
     * @see HashIndexStatsModule
     */

	HashIndexStatsModule retrieveStatsModule();

	/**
     * Gets the index's find count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the find operation's invocation count for this index
     *
     * @see #retrieveStatsModule()
     * @see HashIndexStatsModule#getFindCount(boolean copy)
     */

	double getFindCount();

	/**
     * Gets the index's find duration time attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the find call's duration time for this index in milliseconds
     *
     * @see #retrieveStatsModule()
     * @see HashIndexStatsModule#getFindDurationTime(boolean copy)
     */

	double getFindDurationTime();

	/**
     * Gets the index's result count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the result count for this index and find operation
     *
     * @see #retrieveStatsModule()
     * @see HashIndexStatsModule#getFindResultCount(boolean copy)
     */

    double getFindResultCount();

	/**
     * Gets the index's failure count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the failure count for this index and find operation
     *
     * @see #retrieveStatsModule()
     * @see HashIndexStatsModule#getFindFailureCount(boolean copy)
     */

    double getFindFailureCount();

	/**
     * Gets the index's collision count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the collision count for this index and find operation
     *
     * @see #retrieveStatsModule()
     * @see HashIndexStatsModule#getFindCollisionCount(boolean copy)
     */

    double getFindCollisionCount();

	/**
     * Gets the index's batchupdate count attribute loaded up by the
     * <code>retrieveStatsModule()</code> method.
     *
     * @return the doBatchUpdate method's invocation count for this index
     *
     * @see #retrieveStatsModule()
     * @see MapIndexPlugin#doBatchUpdate(TxID txid, LogSequence sequence)
     * @see HashIndexStatsModule#getBatchUpdateCount(boolean copy)
     */

    double getBatchUpdateCount();

}
