// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2010
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/management/MapMBean.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:21:31
//
// Module  :  MapMBean.java
//
// Source File Description:
//
// Interface for the MBean operations and attributes related to Maps
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d327475      WASX.XD   Nov 29, 2005 joshuad   new code drop
// d329200      WASX.XD   12-6-05      joshuad   gateway enhancements
// b18300       XS 7.1    02/11/2010   jaredp    Added getMapGetCount & getMapHitCount
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.management;

import javax.management.openmbean.TabularData;

import com.ibm.websphere.objectgrid.stats.MapStatsModule;

/**
 * This MBean interface allows a client process to access different attributes
 * and statistical data about a specific map on a server process. In a
 * dynamic ObjectGrid environment, the object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ObjectMap,name=&lt;map&gt;,partition=&lt;partition id&gt;,objectgrid=&lt;objectgrid&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.0.1, XC10
 */
public interface MapMBean {

    /**
     * Gets the <code>MapStatsModule</code> used to retrieve statistics
     * associated with the map for this MBean.
     *
     * @return A MapStatsModule for statistics associated with this map.
     *
     * @see MapStatsModule
     */
    MapStatsModule retrieveStatsModule();

    /**
     * Gets the name of the map associated with this MBean.
     *
     * @return The name of the map.
     */
    String getMapName();

    /**
     * Gets the name of the ObjectGrid containing the map associated with this
     * MBean.
     *
     * @return The name of the ObjectGrid for the map associated with this
     *         MBean.
     */
    String getObjectGridName();

    /**
     * Gets the name of the server containing the replication group member for
     * the map associated with this MBean.
     *
     * @return The name of server containing the replication group member for the
     *         map associated with this MBean.
     */
    String getServerName();

    /**
     * Gets a string representation of the <code>MapStatsModule<code> attributes
     * loaded up by the <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The String form of <code>MapStatsModule</code>
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule
     */
    String getMapStatsModule();

    /**
     * Gets the map count attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The number of entries in the map.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getNumEntries(boolean)
     */
    long getMapCountStatistic();

    /**
     * Gets the hit rate attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The hit rate for the map.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getHitRate(boolean)
     */
    double getMapHitRateStatistic();

    /**
     * Gets the get count attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The get count for the map.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getHitRate(boolean)
     * @since 7.1
     */
    long getMapGetCountStatistic();

    /**
     * Gets the used bytes attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     * <p>
     * The used bytes statistics are accurate only when you are using simple objects
     * or the COPY_TO_BYTES copy mode.
     *
     * @return The number of bytes in use by the map.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getUsedBytes(boolean)
     * @since 7.1
     */
    long getMapUsedBytes();

    /**
     * Gets the hit count attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The hit count for the map.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getHitRate(boolean)
     * @since 7.1
     */
    long getMapHitCountStatistic();

    /**
     * Gets the mean batch update time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The mean batch update time for the map in milliseconds.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getBatchUpdateTime(boolean)
     */
    double getMapBatchUpdateMeanTime();

    /**
     * Gets the maximum batch update time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The maximum batch update time for the map in milliseconds.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getBatchUpdateTime(boolean)
     */
    double getMapBatchUpdateMaxTime();

    /**
     * Gets the minimum batch update time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The minimum batch update time for the map in milliseconds.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getBatchUpdateTime(boolean)
     */
    double getMapBatchUpdateMinTime();

    /**
     * Gets the total batch update time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return The total batch update time for the map in milliseconds.
     *
     * @see #retrieveStatsModule()
     * @see MapStatsModule#getBatchUpdateTime(boolean)
     */
    double getMapBatchUpdateTotalTime();


    /**
     * Retrieves the partition identifier for this map instance.
     * @return The partition identifier.
     * @since WAS XD 6.1.0.4
     */
    int getPartitionId();

    /**
     * Gets the name of the container containing the replication group member for
     * the map associated with this MBean.
     *
     * @return The name of container containing the replication group member for the
     *         map associated with this MBean.
     * @since 8.5
     */
    String getContainerName();

    /**
     * Operation to iterate through all of the entries in this map, convert the key to string form, then match the string against the regular
     * expression if passed, finally return the matching entries. This method could potentially return a very large data structure so care should be
     * taken to ensure the regular expression will reduce the number of keys appropriately.<p>
     *
     * Each CompositeData (row in the TabularData) contains the following items:
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>KeyName</td>
     *     <td>String</td>
     *     <td>The domain name of this ObjectGrid shard.</td>
     * </tr>
     * <tr>
     *     <td>LifetimeIndex</td>
     *     <td>Short</td>
     *     <td>The lifetime index for revisioning.</td>
     * </tr>
     * <tr>
     *     <td>Revision</td>
     *     <td>Long</td>
     *     <td>The revision number of the last update.</td>
     * </tr>
     * </table>
     * @param regex the regular expression to apply to the String form of the key.
     * It should be used in narrowing the entries returned.  If null, all entries are returned.
     * @return A table of entries containing the user readable (String) form of the key and some meta information about the entry.
     * @since 7.1.1
     */
    TabularData retrieveEntries(String regex);
}
