// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.4 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/management/ObjectGridMBean.java, WAS.objectgrid, WASX.XD, b0614.107 2/17/06 20:14:01
//
// Source File Description:
//
// Interface for the MBean operations and attributes related to ObjectGrids
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// d327475      WASX.XD   Nov 29, 2005 joshuad     new code drop
// d329200      WASX.XD   12-6-05      joshuad   gateway enhancements
// d338619      WASX.XD   01/30/06     joshuad   add TPS
// d348005      WXD601.XD 02-16-2006   joshuad   add transaction type
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.management;

import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.TabularData;

import com.ibm.websphere.objectgrid.stats.OGStatsModule;

/**
 * This MBean interface allows a client process to access different attributes
 * and statistical data about a specific ObjectGrid on a server process. In a
 * dynamic ObjectGrid environment, the object name pattern for this MBean is:
 * <pre>
 * com.ibm.websphere.objectgrid:type=ObjectGrid,name=&lt;objectgrid&gt;,mapset=&lt;mapset&gt;,partition=&lt;partition id&gt;,host=&lt;host&gt;,ogServerName=&lt;server&gt;
 * </pre>
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object name.
 *
 * @since WAS XD 6.0.1, XC10
 */
public interface ObjectGridMBean {

    /**
     * Gets the <code>OGStatsModule</code> used to retrieve statistics
     * associated with the ObjectGrid for this MBean.
     *
     * @return an OGStatsModule for statistics associated with this ObjectGrid
     *
     * @see OGStatsModule
     */
    OGStatsModule retrieveStatsModule();


    /**
     * Gets the name of the ObjectGrid associated with this MBean.
     *
     * @return name of the ObjectGrid
     */
    String getObjectGridName();

    /**
     * Gets the name of the server containing the replication group member for
     * the ObjectGrid associated with this MBean.
     *
     * @return the name of server containing the replication group member for the
     *         ObjectGrid associated with this MBean.
     */
    String getServerName();

    /**
     * Gets the name of the container name containing the replication group member for
     * the ObjectGrid associated with this MBean.
     *
     * @return the name of container containing the replication group member for the
     *         ObjectGrid associated with this MBean.
     * @since 8.5
     */
    String getContainerName();

    /**
     * Gets a string representation of the <code>OGStatsModule<code> attributes
     * loaded up by the <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return String form of <code>OGStatsModule</code>
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule
     */
    String getOGStatsModule();

    /**
     * Gets the ObjectGrid count attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the number of transactions
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    long getOGCount();

    /**
     * Gets the maximum transaction time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the maximum transaction time for the ObjectGrid in milliseconds
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    long getOGMaxTranTime();

    /**
     * Gets the minimum transaction time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the minimum transaction time for the ObjectGrid in milliseconds
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    long getOGMinTranTime();

    /**
     * Gets the mean transaction time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the mean transaction time for the ObjectGrid in milliseconds
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    double getOGMeanTranTime();

    /**
     * Gets the total transaction time attribute loaded up by the
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the total transaction time for the ObjectGrid in milliseconds
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    long getOGTotalTranTime();

    /**
     * Transactions per second attribute loaded up by the retrieveStatsModule call.
     * <code>retrieveStatsModule()</code> or
     * <code>refreshStatsModule()</code> method.
     *
     * @return the transactions per second for the ObjectGrid
     *
     * @see #retrieveStatsModule()
     * @see OGStatsModule#getTransactionTime(String, boolean)
     */
    long getOGTransPerSecond();

    /**
     * Retrieves the current revision number of this ObjectGrid shard.
     *
     * @return the current revision number of this ObjectGrid shard.
     * @since 7.1
     */
    long getCurrentRevision();

    /**
     * Retrieves the domain name of this ObjectGrid shard.
     * @return the name of the domain name of this ObjectGrid shard.
     * @since 7.1
     */
    String getDomainName();

    /**
     * Retrieves the lifetime id for this ObjectGrid shard.
     *
     * @return the lifetime id for this ObjectGrid shard.
     * @since 7.1
     */
    String getLifetimeId();

    /**
     * An ObjectGrid shard may exist over several different lifetimes.  As such, each shard instance will have
     * a unique lifetime id and revision number associated with it.  This method returns a TabularData object representing
     * the known history of revision numbers for each lifetime.
     *
     * Each CompositeData (row in the TabularData) contains the following items:
     *
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong></td>
     *     <td><strong><u>Since</u></strong></td>
     * </tr>
     * <tr>
     *     <td>Domain</td>
     *     <td>String</td>
     *     <td>The domain name of this ObjectGrid shard.</td>
     *     <td>7.1</td>
     * </tr>
     * <tr>
     *     <td>Server</td>
     *     <td>String</td>
     *     <td>The name of the server owning the lifetime id.</td>
     *     <td>8.5</td>
     * </tr>
     * <tr>
     *     <td>LifetimeId</td>
     *     <td>String</td>
     *     <td>The lifetime id of this ObjectGrid shard.</td>
     *     <td>7.1</td>
     * </tr>
     * <tr>
     *     <td>Revision</td>
     *     <td>Long</td>
     *     <td>The revision of this ObjectGrid shard.</td>
     *     <td>7.1</td>
     * </tr>
     * </table>
     *
     * @return TabularData representing the known lifetimes and revisions of this shard.
     * @throws OpenDataException
     * @see TabularData
     * @since 7.1
     */
    TabularData getKnownRevisions();


    /**
     * Get the shard's list of foreign or domestic linked primaries.<p>
     *
     * This method returns a TabularData object representing the current state of each primary shard link.<p>
     *
     * Each CompositeData (row in the TabularData) contains the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>RemoteDomain</td>
     *     <td>String</td>
     *     <td>The catalog service domain name of the remote primary shard..</td>
     * </tr>
     * <tr>
     *     <td>RemoteContainer</td>
     *     <td>String</td>
     *     <td>The container name of the remote primary shard.</td>
     * </tr>
     * <tr>
     *     <td>Status</td>
     *     <td>String</td>
     *     <td>The status of the link.  Valid states include: <code>online</code> and <code>recovery</code>.</td>
     * </tr>
     * </table>
     * @return TabularData representing the linked primaries
     * @since 7.1.1
     */
    TabularData getPrimaryShardLinks();


}
