// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2010-2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// $Id$
//
// Module  :  PlacementMediationServiceMBean.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason    Version   Date         Userid     Change Description
// --------- --------- ------------ ---------- -----------------------------------------
// b18175    7.1       04/27/2010   vandersn   new MBean for connection and disconnecting domains
// b9434     8.6       05/01/2012   kristip    Add an additional getLinkedDomains mbean method

package com.ibm.websphere.objectgrid.management;

import javax.management.openmbean.CompositeData;
import javax.management.openmbean.TabularData;

import com.ibm.websphere.objectgrid.server.CatalogServerProperties;


/**
 * This MBean interface allows a client process to perform operations on and get status from the PlacementMediationService
 * running in a dynamic environment. The object name pattern for this MBean is:
 *
 * <pre>
 * com.ibm.websphere.objectgrid:type=PlacementMediationService
 * </pre>
 *
 * If ObjectGrid is running in a WebSphere Application Server process, more key=value pairs may be added to the object
 * name.
 *
 * @since 7.1, XC10
 */
public interface PlacementMediationServiceMBean {

    /**
     * Establish a link between this domain and the foreign domain specified.  This is functionally equivalent to providing
     * the foreign domain and its end points in the server properties file at server startup time.<p>
     *
     * Domains that are linked will share placement with each other.  When compatible map sets are detected within
     * linked domains, a multi-primary topology will be achieved.  Data written to a primary is either domain will be asynchronously
     * replicated to the other domain.<p>
     *
     * The result is a CompositeData that includes the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Result</td>
     *     <td>String</td>
     *     <td>The result of the attempt.</td>
     * </tr>
     * <tr>
     *     <td>StatusBefore</td>
     *     <td>String</td>
     *     <td>The status of the link before the attempt was made to establish the link.</td>
     * </tr>
     * <tr>
     *     <td>StatusAfter</td>
     *     <td>String</td>
     *     <td>The status of the link after the attempt was made to establish the link.</td>
     * </tr>
     * </table>
     *
     * @param foreignDomain
     *            the name of the foreign domain
     * @param endPoints
     *            end points of the foreign domain
     * @return CompositeData representing the status of the attempt to link with the foreign domain
     *
     * @see CatalogServerProperties#setForeignDomains(String)
     * @see CatalogServerProperties#setDomainEndPoints(String, String)
     */
    public CompositeData establishLink(String foreignDomain, String endPoints);

    /**
     * Dismiss a previously established link with the foreign domain specified.  Any map sets that were participating
     * in a multi-primary topology will be disconnected from each other.  Data will no longer be replicated from
     * between domains.<p>
     *
     * The result is a CompositeData that includes the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Result</td>
     *     <td>String</td>
     *     <td>The result of the attempt.  Can be one of:  SUCCESS, FAILURE, NOP</td>
     * </tr>
     * <tr>
     *     <td>StatusBefore</td>
     *     <td>String</td>
     *     <td>The status of the link before the attempt was made to dismiss the link.  Can be one of:
     *     LINKED, ESTABLISHING_LINK, UNLINKED, DISMISSING_LINK</td>
     * </tr>
     * <tr>
     *     <td>StatusAfter</td>
     *     <td>String</td>
     *     <td>The status of the link after the attempt was made to dismiss the link.  Can be one of:
     *     LINKED, ESTABLISHING_LINK, UNLINKED, DISMISSING_LINK</td>
     * </tr>
     * </table>
     *
     * @param foreignDomain
     *            the name of the foreign domain
     * @return CompositeData representing the status of the attempt to dismiss the link with the foreign domain
     */
    public CompositeData dismissLink(String foreignDomain);

    // fix-vandersn: more description on the CompositeData returned above.

    /**
     * Retrieve the foreign domains that have an active link with the local domain.<p>
     *
     * The result is a TabularData where each row is a CompositeData that includes the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Domain</td>
     *     <td>String</td>
     *     <td>The name of the foreign domain linked to the local domain.</td>
     * </tr>
     * </table>
     *
     * @return TabularData representing the foreign domains linked to this domain
     */
    public TabularData getLinkedDomains();
    
    /**
     * Retrieve the foreign domains that have an active link with the local domain and the map sets eligible for linking.<p>
     *
     * The result is a TabularData where each row is a CompositeData that includes the following items:
     * <table>
     * <tr>
     *     <td><strong><u>Item Name</u></strong><td><strong><u>Type</u></strong><td><strong><u>Description</u></strong>
     * </tr>
     * <tr>
     *     <td>Domain</td>
     *     <td>String</td>
     *     <td>The name of the foreign domain linked to the local domain.</td>
     * </tr>
     * <tr>
     *     <td>ObjectGrid</td>
     *     <td>String</td>
     *     <td>The name of the ObjectGrid that is compatible with the foreign domain.</td>
     * </tr>
     * <tr>
     *     <td>MapSet</td>
     *     <td>String</td>
     *     <td>The name of the map set that is compatible with the foreign domain.</td>
     * </tr>
     * </table>
     *
     * @return TabularData representing the foreign domains and eligible map sets linked to this domain
     * @since 8.6
     */
    public TabularData getLinkedDomainsWithGrids();

    // fix-vandersn: method for get replicated map sets
}
