// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.7 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/CacheEntry.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:22:16
//
// Module  :  CacheEntry.java
//
// Source File Description:
//
//    see javadoc for this interface.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   02/17/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/21/2005   sutter    M2 Deliverable
// XD60_M3_B.8  WASX.XD   04/07/2005   sutter    M3 Client/Server updates
// 278516       WASX.XD   05/26/2005   kjlaw     add get/put slot methods and SLOT_NAME.
// 281081       WASX.XD   06/07/2005   kjlaw     backed out 278516 changes.
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.io.Serializable;
import java.util.Collection;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.plugins.io.KeySerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.ValueSerializerPlugin;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedEntry;
import com.ibm.websphere.objectgrid.plugins.io.dataobject.SerializedKey;

/**
 * This interface represents a cache entry in an ObjectGrid map.
 *
 * @since WAS XD 6.0, XC10
 */
public interface CacheEntry extends Serializable
{
    /**
     * Indicates whether this entry is in the <code>BackingMap</code>
     *
     * @return Returns <code>true</code> if this element is in the BackingMap
     */
    public boolean isInBackingMap();

    /**
     * Returns the key for this entry.
     * <p>
     * For a CacheEntry on an {@link ObjectMap} that is configured to use a {@link KeySerializerPlugin}, the
     * value will be a {@link SerializedKey} object. If required, you
     * can use the {@link SerializedEntry#getObject()} method to retrieve (possibly inflating
     * the serialized object) the original key object.
     *
     * @return the key
     */
    Object getKey();

    /**
     * Returns the committed value for this entry.
     * <p>
     * The type of the object returned from the <code>getCommittedValue()</code> method depends on various
     * configuration and storage options used by the {@link ObjectMap} that holds the
     * CacheEntry. In the default case, <code>getCommittedValue()</code> returns the Java object
     * of the same type that was put into the map. For an {@link ObjectMap} that is configured
     * to use a {@link ValueSerializerPlugin}, the committed value depends on the underlying storage
     * mechanism, typically represented as an array of bytes.
     *
     * @return the committed value
     */
    Object getCommittedValue();

    /**
     * Returns the time-to-live value for this entry.
     *
     * @return the time-to-live value
     */
    long getTTL();

    /**
     * Returns the last time this entry was accessed.
     *
     * @return last access time.
     */
    long getLastAccessTime();
}