// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.8 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/plugins/LogSequence.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 17:23:01
//
// Module  :  LogSequence.java
//
// Source File Description:
//
//    Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M1_B.1  WASX.XD   01/14/2005   sutter    new code drop
// XD60_M2_B.2  WASX.XD   03/21/2005   sutter    M2 Deliverable
// XD60_M3_B.10 WASX.XD   04/07/2005   sutter    Re-factor datagrid/datagrid.impl
// XD60_M3_B.8  WASX.XD   04/11/2005   sutter    Client/Server updates
// 272056       WASX.XD   05/03/2005   jhanders  combine changes() and evictees()
// 271911       WASX      May 16, 2005 sutter    Partitioning support updates
// XD60_M4_B.21 WASX.XD   05/19/2005   sutter    Client/server optimizations
// XD60_M4_B.17 WASX.XD   05/25/2005   sutter    ObjectGrid plan reset (remove some public APIs)
// 284443       WASX.XD   06/22/2005   sutter    changed getLoaderChanges to getPendingChanges
// 294637.1     WASX.XD   08/09/2005   kjlaw     add isRollback method.
// XD601_M2_B.2 WASX.XD   09/27/2005   sutter    added getObjectGridName method
// ------------ --------- ------------ --------- -----------------------------------------

package com.ibm.websphere.objectgrid.plugins;

import java.io.Serializable;
import java.util.Collection;
import java.util.Iterator;

import com.ibm.websphere.objectgrid.ObjectMap;
import com.ibm.websphere.objectgrid.Session;

/**
 * LogSequence is the ordered list of changes performed against a given map
 * for a given transaction. These changes are recorded as LogElement objects.
 *
 * @since WAS XD 6.0, XC10
 */
public interface LogSequence extends Serializable
{
    /**
     * Returns the total number of LogElements within this LogSequence.
     *
     * @return total number of LogElements
     */
    int size();

    /**
     * Returns an iterator for processing all of the "pending" changes for a
     * LogSequence (for example, pending inserts, updates, and deletes).
     * <p>
     * This method is normally used by a <code>Loader</code>.  A pending change
     * is one that has not been written out to a loader yet using a
     * <code>flush()</code> operation. Note, the returned iterator's
     * <code>remove()</code> is not allowed to be called and will throw
     * an exception.
     *
     * @return an Iterator for processing the pending LogElement changes
     *
     * @see ObjectMap#flush()
     * @see Session#flush()
     */
    Iterator getPendingChanges();

    /**
     * Returns an iterator for processing all of the changes for a LogSequence.
     * <p>
     * This method would normally be used by an <code>Evictor</code> and other
     * plugins that want to know all of the changes introduced by this LogSequence.
     * Note, the returned iterator's <code>remove()</code> is not allowed to be
     * called and will throw an exception.
     *
     * @return an Iterator for processing all of the LogElement changes
     */
    Iterator getAllChanges();

    /**
     * Returns an iterator for processing the LogElements that are of the
     * requested type.
     * <p>
     * Each member of the input Collection should be one of the defined
     * LogElement Types (INSERT, UPDATE, DELETE, FETCH, TOUCH, or EVICT).
     * Note, the returned iterator's <code>remove()</code> is not allowed to be
     * called and will throw an exception.
     *
     * @param types A Collection of LogElement Types (INSERT, UPDATE, etc)
     *
     * @return Iterator for processing all LogElements that support the input Type(s)
     *
     * @throws IllegalArgumentException if types is <code>null</code>
     *
     * @see LogElement#DELETE
     * @see LogElement#EVICT
     * @see LogElement#FETCH
     * @see LogElement#INSERT
     * @see LogElement#TOUCH
     * @see LogElement#UPDATE
     * @see LogElement#CLEAR
     */
    Iterator getChangesByTypes(Collection /* <LogElement.Type> */ types);

    /**
     * Returns an iterator for processing the LogElements that have the
     * requested keys.
     * <p>
     * Note, the returned iterator's <code>remove()</code> is not allowed to be
     * called and will throw an exception.
     *
     * @param keys a collection of key objects
     * @return an Iterator for processing all LogElements that match the input key(s)
     */
    Iterator getChangesByKeys(Collection /* <Object> */ keys);

    // TODO Not considered necessary for XD v6.  Removed from the interface, but left in
    // the LogSequenceImpl.

    // /**
    //  * This method will return an iterator for processing the LogElements
    //  * that do not match the Type(s) as defined in the input parameter.  Each
    //  * member of the input Collection should be one of the defined
    //  * LogElement Types (INSERT, UPDATE, DELETE, FETCH, TOUCH, or EVICT).
    //  *
    //  * @param types A Collection of LogElement Types (INSERT, UPDATE, etc)
    //  * @return an Iterator for processing all LogElements that do not support the input Type(s)
    //  */
    // Iterator getChangesWithoutTypes(Collection /* <LogElement.Type> */ types);
    //
    // /**
    //  * This method will return an iterator for processing the LogElements
    //  * that do not match the keys as defined in the input parameter.
    //  *
    //  * @param keys A collection of key objects
    //  * @return Iterator for processing all LogElements that do not match the input key(s)
    //  */
    // Iterator getChangesWithoutKeys(Collection /* <Object> */ keys);
    //
    /**
     * Returns the name of the map that these changes apply to.
     * <p>
     * The caller can use the return value of this method as input to the
     * <code>Session.getMap(String)</code> method.
     *
     * @return The name of the map that these changes apply to
     *
     * @see Session#getMap(String)
     */
    String getMapName();

    /**
     * Returns the name of the ObjectGrid that houses the map that
     * these changes apply to.
     *
     * @return The name of the ObjectGrid that this LogSequence is associated with
     *
     * @since WAS XD 6.0.1
     */
    String getObjectGridName();

    // TODO Not considered necessary for XD v6.  Removed from the interface, but left in
    // the LogSequenceImpl.

    // /**
    //  * This method will be used to partition a LogSequence based on the
    //  * partitioning algorithm for the Map.
    //  *
    //  * @param pm PartitionManager instance that will determine the partition associated with a key
    //  * @return List of partitioned LogSequences.  The first LogSequence in the List
    //  * corresponds to the first partition, etc.
    //  */
    // List /*<LogSequence>*/ partition(PartitionManager pm);

    /**
     * Returns whether this LogSequence has any LogElements that
     * would "dirty" a Map.
     * <p>
     * That is, if it contains any LogElements of any type other than Fetch/Get,
     * it is considered "dirty".
     *
     * @return <code>true</code> if the LogSequence would modify a Map, if applied;
     *         <code>false</code> if the LogSequence would not modify a Map, if applied
     */
    boolean isDirty();

    /**
     * Returns whether or not this LogSequence was generated to rollback
     * a transaction.
     * <p>
     * Note, depending on when this LogSequence is used, the transaction itself
     * might already be rolled back.
     *
     * @return true iff this LogSequence was generated to rollback a transaction.
     *
     * @since WAS XD 6.0.1
     */
    boolean isRollback();
}
