// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005,2006
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.8 XD/ws/code/objectgrid/src/com/ibm/websphere/objectgrid/security/config/ClientSecurityConfiguration.java, WAS.objectgrid, WASX.XD, b0614.107 2/24/06 13:00:54
//
// Module  :  ClientSecurityConfiguration.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD60_M3_B.13 WASX.XD   04/08/2005   jitang    ObjectGrid - JAAS / Security - Drop 1 - Basic Support
// PK17543      WASX.XD   01/26/2006   jitang    Update JavaDoc
// d344266      WXD601.XD 02/08/2006   jitang    Add getters for credentialGenerator class and props
//
// ------------ ---------- ------------ -------------------------------------------------
package com.ibm.websphere.objectgrid.security.config;

import java.io.Serializable;
import java.net.URL;

import com.ibm.websphere.objectgrid.ObjectGridManager;
import com.ibm.websphere.objectgrid.security.SecurityConstants;
import com.ibm.websphere.objectgrid.security.plugins.CredentialGenerator;

/**
 * <p>This interface represents the client side security configurations.
 * User can call ClientSecurityConfigurationFactory.getClientSecurityConfiguration()
 * to get a ClientSecurityConfiguration instance.
 * 
 * <p>Users are not recommended to implement this interface in case more methods
 * are added in future releases.
 * 
 * @since WAS XD 6.0.1, XC10
 * 
 * @see ObjectGridManager#connect(ClientSecurityConfiguration, URL)
 * @see ObjectGridManager#connect(String, ClientSecurityConfiguration, URL)
 */
public interface ClientSecurityConfiguration extends Serializable {

    /**
     * Gets the authentication retry count for this client security configuration.
     * <p>
     * The authentication retry count is the number a times authentication
     * will be reattempted if there is a failure during authentication indicating 
     * the credential is expired. 
     * 
     * @return   the argument that was passed to the 
     *           <code>setAuthenticationRetryCount(int)</code> method of
     *           this interface or <code>0</code> if <code>setAuthenticationRetryCount</code> 
     *           was not previously called for this object.
     * 
     * @see #setAuthenticationRetryCount(int)
     */
    int getAuthenticationRetryCount();

    /**
     * Sets the authentication retry count for this client security configuration.
     *  
     * @param authenticationRetryCount the authentication retry count
     * 
     * @throws IllegalArgumentException if authenticationRetryCount is less than zero
     */
    void setAuthenticationRetryCount(int authenticationRetryCount);

    /**
     * Gets the transport type for this client security configuration.
     * <p>
     * The return value will be one of the three transport types defined
     * on the <code>SecurityConstants</code> class.
     * 
     * @return   the argument that was passed to the 
     *           <code>setTransportType(int)</code> method of this interface or 
     *           <code>SecurityConstants.SSL_SUPPORTED</code> if <code>setTransportType</code>  
     *           was not previously called for this object.
     *
     * @see #setTransportType(int) 
     * @see SecurityConstants#TCP_IP
     * @see SecurityConstants#SSL_SUPPORTED
     * @see SecurityConstants#SSL_REQUIRED
     */
    int getTransportType();

    /**
     * Gets the client certificate authentication type for this client
     * security configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setClientCertificateAuthentication(int)</code> method of
     *           this interface or <code>SecurityConstants.CLIENT_CERTIFICATE_AUTHENTICATION_SUPPORTED</code> 
     *           if <code>setClientCertificateAuthentication</code> was not  
     *           previously called for this object.
     * 
     * @see #setClientCertificateAuthentication(int)
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_NEVER
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_SUPPORTED
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_REQUIRED
     */
    public int getClientCertificateAuthentication();

    /**
     * Gets whether security is enabled for this client security configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setSecurityEnabled(boolean)</code> method of
     *           this interface or <code>false</code> 
     *           if <code>setSecurityEnabled</code> was not  
     *           previously called for this object.
     * 
     * @see #setSecurityEnabled(boolean)
     */
    boolean isSecurityEnabled();

    /**
     * Gets whether single sign on is enabled for this client security configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setSingleSignOnEnabled(boolean)</code> method of
     *           this interface or <code>false</code> 
     *           if <code>setSingleSignOnEnabled</code> was not  
     *           previously called for this object.
     *
     * @see #setSingleSignOnEnabled(boolean)
     */
    boolean isSingleSignOnEnabled();

    /**
     * Set the client certificate authentication type for this client
     * security configuration.
     * 
     * @param clientCertAuthen the client ceritificate authentication type
     * 
     * @throws IllegalArgumentException if clientCertAuthen is not one of
     *         the valid values from the <code>SecurityConstants</code> class.
     * 
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_NEVER
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_SUPPORTED
     * @see SecurityConstants#CLIENT_CERTIFICATE_AUTHENTICATION_REQUIRED
     */
    void setClientCertificateAuthentication(int clientCertAuthen);

    /**
     * Sets the transport type for this client security configuration.
     * 
     * @param type the transport type
     * 
     * @throws IllegalArgumentException if type is not one of the valid
     *         values from the <code>SecurityConstants</code> class.
     * 
     * @see SecurityConstants#TCP_IP
     * @see SecurityConstants#SSL_SUPPORTED
     * @see SecurityConstants#SSL_REQUIRED
     */
    void setTransportType(int type);

    /**
     * Sets whether or not security is enabled for this client security
     * configuration.
     * 
     * @param isSecurityEnabled whether or not security is enabled
     */
    void setSecurityEnabled(boolean isSecurityEnabled);

    /**
     * Sets whether or not single sign on is enabled for this client security
     * configuration.
     *   
     * @param enabled whether or not single sign on is enabled
     */
    void setSingleSignOnEnabled(boolean enabled);

    /**
     * Gets the <code>SSLConfiguration</code> object associated with this client
     * security configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setSSLConfiguration(SSLConfiguration)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setSSLConfiguration</code> was not  
     *           previously called for this object.
     * 
     * @see #setSSLConfiguration(SSLConfiguration)
     * @see SSLConfiguration
     */
    SSLConfiguration getSSLConfiguration();

    /**
     * Sets the <code>SSLConfiguration</code> object associated with this client
     * security configuration.
     * 
     * @param sslConfig the SSL setting for this object.
     * 
     * @see SSLConfiguration
     */
    void setSSLConfiguration(SSLConfiguration sslConfig);

    /**
     * Gets the credential authentication type for this client security
     * configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setCredentialAuthenticationType(int)</code> method of
     *           this interface or <code>SecurityConstants.CREDENTIAL_AUTHENTICATION_SUPPORTED</code> 
     *           if <code>setCredentialAuthenticationType</code> was not  
     *           previously called for this object.
     *
     * @see #setCredentialAuthenticationType(int) 
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_NEVER
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_SUPPORTED
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_REQUIRED
     */
    int getCredentialAuthenticationType();

    /**
     * Sest the credential authentication type for this client security
     * configuration.
     *  
     * @param credAuthen the credential authentication type
     * 
     * @throws IllegalArgumentException if credAuthen is not one of
     *         the valid values from the <code>SecurityConstants</code> class.
     * 
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_NEVER
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_SUPPORTED
     * @see SecurityConstants#CREDENTIAL_AUTHENTICATION_REQUIRED
     */
    void setCredentialAuthenticationType(int credAuthen);

    /**
     * Sets the <code>CredentialGenerator</code> object for this client
     * security configuration.
     * 
     * @param generator a CredentialGenerator object  
     * 
     * @see CredentialGenerator  
     */
    void setCredentialGenerator(CredentialGenerator generator);

    /**
     * Gets the <code>CredentialGenerator</code> object for this client
     * security configuration.
     * 
     * @return   the argument that was passed to the 
     *           <code>setCredentialGenerator(CredentialGenerator)</code> method of
     *           this interface or <code>null</code> 
     *           if <code>setCredentialGenerator</code> was not  
     *           previously called for this object.
     *
     * @see #setCredentialGenerator(CredentialGenerator) 
     * @see CredentialGenerator  
     */
    CredentialGenerator getCredentialGenerator();

    /**
     * Gets the <code>CredentialGenerator</code> implementation class name set 
     * in the client security property file. 
     * <p>
     * If the "credentialGeneratorClass" property is not 
     * set in the security property file, a <code>null</code> will be returned.
     * 
     * @return the value of the "credentialGeneratorClass" from the client 
     *         security property file. <code>null</code> is returned if no value
     *         is set for this property.
     * 
     * @see #getCredentialGeneratorProps() 
     */
    String getCredentialGeneratorClass();

    /**
     * Gets the <code>CredentialGenerator</code> properties set in the client security 
     * property file. 
     * <p>
     * If the "credentialGeneratorProps" property is not set in the security 
     * security property file, a <code>null</code> will be returned. 
     * <p>
     * If the "credentialGeneratorProps" property is set, but the property 
     * "credentialGeneratorClass" is not set, the "credentialGeneratorProps"
     * property will be ignored. In this case, a <code>null</code> will be 
     * returned by this method.  
     * 
     * @return the value of the "credentialGeneratorProps" property from 
     *         the client security property file. <code>null</code> is returned
     *         if no value is set for this property or for the 
     *         "credentialGeneratorClass" property.
     * 
     * @see #getCredentialGeneratorClass()
     */
    String getCredentialGeneratorProps();
}