// IBM Confidential OCO Source Material
// 5724-J34 (C) COPYRIGHT International Business Machines Corp. 2005-2012
// The source code for this program is not published or otherwise divested
// of its trade secrets, irrespective of what has been deposited with the
// U.S. Copyright Office.
//
// 1.3 XD/ws/code/objectgrid.client/src/com/ibm/websphere/objectgrid/security/plugins/builtins/WSTokenCredential.java, WAS.objectgrid, WASX.XD, b0614.107 2/14/06 18:22:07
//
// Module  :  WSTokenCredential.java
//
// Source File Description: 
//
//  Need a file description.
//
// Change Activity:
//
// Reason       Version   Date         Userid    Change Description
// ------------ --------- ------------ --------- -----------------------------------------
// XD601_M3_Bx  WASX.XD   11/15/2005   jitang    ObjectGrid security support
// d335512      WASX.XD   01/04/2006   jitang    push both AT and OAT in the WS credential generator
// PK17543      WASX.XD   01/11/2006   jitang    Update JavaDoc
// rtc8700      WXS8.5    03/23/2012   jhanders  Update hashcode and equals to be spec compliant to avoid memory leak
// ------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.objectgrid.security.plugins.builtins;

import java.util.Arrays;

import com.ibm.websphere.objectgrid.security.plugins.Credential;

/**
 * This class represents a WebSphere Application Server security token credential.
 * An instance of this class contains byte arrays which represent the security tokens used 
 * in the Application server. Specifically, this credential contains two tokens, 
 * a WebSphere specific authentication token and a WebSphere specific opaque 
 * authorization token. 
 * 
 * @since WAS XD 6.0.1, XC10
 * 
 * @see WSTokenCredentialGenerator#getCredential()
 */
public final class WSTokenCredential implements Credential {

    private static final long serialVersionUID = 3588604961012291551L;

    /** the authentication token byte array*/
    private final byte[] ivAT;

    /** the opaque authorization token byte array */
    private final byte[] ivOAT;

    /**
     * Creates a new WebSphere token credential with the specified security
     * tokens.
     * 
     * @param at the authentication token
     * @param oat the opaque authorization token
     */
    public WSTokenCredential(byte[] at, byte[] oat) {
        ivAT = at;
        ivOAT = oat;
    }

    /**
     * Gets the authetication token for this credential.
     * 
     * @return   the authentication token argument that was passed to the 
     *           constructor of this class 
     */
    public byte[] getAT() {
        return ivAT;
    }

    /**
     * Gets the opaque authorization token for this credential.
     * 
     * @return   the opaque authorization token argument that was passed to the 
     *           constructor of this class 
     */
    public byte[] getOAT() {
        return ivOAT;
    }

    /**
     * Checks two WSTokenCredential objects for equality.
     * <p>
     * Two WSTokenCredential objects are equal if and only if their  
     * authentication tokens and opaque authorization tokens are equal.
     * 
     * @param obj the object we are testing for equality with this object.
     *
     * @return <code>true</code> if both WSTokenCredential objects are equivalent.
     * 
     * @see Credential#equals(Object)
     */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		WSTokenCredential other = (WSTokenCredential) obj;
		if (!Arrays.equals(ivAT, other.ivAT))
			return false;
		if (!Arrays.equals(ivOAT, other.ivOAT))
			return false;
		return true;
	}

    /**
     * Returns the hashcode of the WSTokenCredential object.
     * 
     * @return the hash code of this object
     * 
     * @see Credential#hashCode()
     */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(ivAT);
		result = prime * result + Arrays.hashCode(ivOAT);
		return result;
	}
}