//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

import javax.naming.NamingException;
import javax.naming.Reference;
import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.cci.Connection;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.ConnectionSpec;
import javax.resource.cci.RecordFactory;
import javax.resource.cci.ResourceAdapterMetaData;

import com.ibm.ws.xs.ra.cci.XSConFactoryImpl;
/**
 * The XSConnectionFactory creates connections to eXtreme Scale ObjectGrids.<p>
 * 
 * Usage example:<p>
 * <pre>
 * InitialContext ctx = new InitialContext();
 * XSConnectionFactory cf = (XSConnectionFactory) ctx.lookup("java:comp/env/wxsconnection");
 * XSConnection con = cf.getConnection("MyGrid");
 * Session ogSession = con.getSession();
 * ...
 * 
 * con.close();
 * 
 * </pre>
 * For additional examples, see the <a href="package.html"> package documentation</a>.
 * @since 8.5
 * @since XC10
 */
public final class XSConnectionFactory 
implements javax.resource.cci.ConnectionFactory {

	private static final long serialVersionUID = 3174169865963425559L;
	
	private XSConFactoryImpl xsCF;
	
	/**
	 * Sets the Reference instance
	 * @param ref - Reference instance
	 * @see javax.resource.cci.ConnectionFactory#setReference(Reference)
	 */
	public void setReference(Reference ref) {
		xsCF.setReference(ref);
	}

	/**
	 * Gets the Reference instance
	 * @return Reference instance
	 * @see javax.resource.cci.ConnectionFactory#getReference()
	 */
	public Reference getReference() throws NamingException {
		return xsCF.getReference();
	}

	/**
     * Retrieve an XSConnection for the ObjectGrid configured on the ConnectionFactory.
     * @return the connection to the eXtreme Scale ObjectGrid
     * @see ConnectionFactory#getConnection()
     * @throws ResourceException
     */
	public Connection getConnection() throws ResourceException {
		return xsCF.getConnection();
	}

	/**
     * Retrieve an XSConnection for the ObjectGrid using the specified XSConnectionSpec properties.
     * 
     * @param spec the {@link XSConnectionSpec} properties used to retrieve the correct connection. 
     * @return the connection to the eXtreme Scale ObjectGrid
     * @see ConnectionFactory#getConnection(ConnectionSpec)
     * @see XSConnectionSpec
     * @throws ResourceException
     */
	public Connection getConnection(ConnectionSpec spec)
			throws ResourceException {
		return xsCF.getConnection(spec);
	}
	
	/**
     * Retrieve an XSConnection for the specified ObjectGrid name.
     * @param gridName - The ObjectGrid name
     * @return the connection to the eXtreme Scale ObjectGrid
     * @throws ResourceException
     */
	public Connection getConnection(String gridName) throws ResourceException {
		return xsCF.getConnection(gridName);
	}

	/**
	 * Retrieve the metadata information regarding the eXtreme Scale resource adapter
	 * @return The eXtreme Scale resource adapter metadata
	 * @see javax.resource.cci.ConnectionFactory#getMetaData()
	 * @throws ResourceException
	 */
	public ResourceAdapterMetaData getMetaData() throws ResourceException {
		return xsCF.getMetaData();
	}

	/**
	 * Retrieve the RecordFactory instance
	 * @see javax.resource.cci.ConnectionFactory#getRecordFactory()
	 * @throws NotSupportedException
	 */
	public RecordFactory getRecordFactory() throws ResourceException {
		return xsCF.getRecordFactory();
	}

    /**
     * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
     * The reflection lookup is called from:
     *   - com.ibm.ws.xs.ra.spi.XSMConFactoryImpl.createXSFactory()
     */
	@SuppressWarnings("unused")
	private void setImpl(XSConFactoryImpl cf) {
		this.xsCF = cf;
	}
	
    /**
     * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
     * The reflection lookup is called from:
     *   - Unit test project wxs.ra.unittest.web/CommonTester
     */
	@SuppressWarnings("unused")
	private XSConFactoryImpl getImpl() {
		return this.xsCF;
	}
	
    public String toString() {
        return super.toString() + "[" + (xsCF == null ? "none" : xsCF.toString() ) + "]";   
    }
    
    /**
     * Get the eXtreme Scale specific property: CatalogDomain
     * @return The catalog service domain name defined in WebSphere Application Server
     */
    public String getCatalogServiceDomain() {
        return xsCF.getManagedConnectionFactory().getCatalogServiceDomain();
    }
    /**
     * Get the eXtreme Scale specific property: CatalogServiceEndpoints
     * @return The catalog service domain end points
     */
    public String getCatalogServiceEndpoints() {
        return xsCF.getManagedConnectionFactory().getCatalogServiceEndpoints();
    }
    /**
     * Get the eXtreme Scale specific property: ClientPropertiesResource
     * @return The resource path of the client properties file
     */
    public String getClientPropertiesResource() {
        return xsCF.getManagedConnectionFactory().getClientPropertiesResource();
    }
    /**
     * Get the eXtreme Scale specific property: ClientPropertiesURL
     * @return The URL of the client properties file
     */
    public String getClientPropertiesURL() {
        return xsCF.getManagedConnectionFactory().getClientPropertiesURL();
    }
    /**
     * Get the eXtreme Scale specific property: ConnectionName
     * @return The name of the eXtreme Scale client connection
     */
    public String getConnectionName() {
        return xsCF.getManagedConnectionFactory().getConnectionName();
    }
    /**
     * Get the eXtreme Scale specific property: ObjectGridName
     * @return The data grid name
     */
    public String getObjectGridName() {
        return xsCF.getManagedConnectionFactory().getObjectGridName();
    }
    /**
     * Get the eXtreme Scale specific property: ObjectGridResource
     * @return The resource path of the client data grid override XML file
     */
    public String getObjectGridResource() {
        return xsCF.getManagedConnectionFactory().getObjectGridResource();
    }
    /**
     * Get the eXtreme Scale specific property: ObjectGridURL
     * @return The URL of the client data grid override XML file
     */
    public String getObjectGridURL() {
        return xsCF.getManagedConnectionFactory().getObjectGridURL();
    }
    
    /**
     * Get the eXtreme Scale specific property: LocalGrid
     * @return Is this managed connection factory used only for access to a local ObjectGrid instance?
     */
    public boolean isLocalGrid() {
        return xsCF.getManagedConnectionFactory().isLocalGrid();
    }
}
