//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

import javax.resource.cci.ConnectionSpec;

import com.ibm.websphere.objectgrid.ObjectGrid;
import com.ibm.websphere.objectgrid.Session;
import com.ibm.websphere.objectgrid.Session.TxCommitProtocol;
import com.ibm.websphere.objectgrid.security.plugins.CredentialGenerator;
import com.ibm.ws.objectgrid.ObjectGridImpl;


/**
 * The ConnectionSpec for retrieving XSConnections from an XSConnectionFactory
 * @since 8.5
 * @since XC10
 * @see XSConnectionFactory
 */
public final class XSConnectionSpec 
implements ConnectionSpec, Cloneable {

    private boolean beginNoWriteThrough = false;

    private String objectGridName = null;
    
    private CredentialGenerator credGen = null;
        
    private ObjectGrid og = null;
    
    private boolean multiPartitionSupportEnabled = false;    
    
    /**
     *  Creates a default XSCOnnectionSpec instance.
     */
    public XSConnectionSpec() {
        super();
    }

    /**
     * Returns flag indicating whether the Session transaction should be started using 
     * using the <code>Session.beginNoWriteThrough()</code> method.
     * 
     * @return flag indicating whether the Session transaction should be started using 
     * using the <code>Session.beginNoWriteThrough()</code> method.
     */
    public boolean isBeginNoWriteThrough() {
        return beginNoWriteThrough;
    }

    /**
     * Set the boolean flag indicating whether the Session transaction should be started using 
     * using the <code>Session.beginNoWriteThrough()</code> method.
     *
     * @param beginNoWriteThrough
     * @see Session#begin()
     * @see Session#beginNoWriteThrough()
     */
    public void setBeginNoWriteThrough(boolean beginNoWriteThrough) {
        this.beginNoWriteThrough = beginNoWriteThrough;
    }

    /**
     * Returns the object grid name for the connection.
     * @return object grid name
     */
    public String getObjectGridName() {
        return objectGridName;
    }

    /**
     * Sets the object grid name to be used for the connection.
     * @param objectGridName object grid name to be used for the connection
     */
    public void setObjectGridName(String objectGridName) {
        this.objectGridName = objectGridName;
    }

    /**
     * Retrieve the local object grid instance used for the connection, if set.
     * @return null if the connection retrieves its ObjectGrid instance normally
     *   from the physical connection, or non-null if {@link #setLocalObjectGrid(ObjectGrid)} was specified.
     * @since 8.6
     */
    public ObjectGrid getLocalObjectGrid() {
        return this.og;
    }
    
    /**
     * Sets a local ObjectGrid instance to be used for connections. The
     * local ObjectGrid instance overrides the standard behavior and allows
     * a physical connection to be used with a specific local ObjectGrid instance, such 
     * as a shard.
     * @param instance the local ObjectGrid instance.
     * @since 8.6
     */
    public void setLocalObjectGrid(ObjectGrid instance) {
        this.og = instance;
    }
    
    /**
     * Returns the <code>CredentialGenerator</code> object for this client
     * security for the connection.
     * 
     * @return   the argument that was passed to the 
     *           <code>setCredGen(CredentialGenerator)</code> method of
     *           this object or <code>null</code> 
     *           if <code>setCredGen</code> was not  
     *           previously called for this object.
     */
    public CredentialGenerator getCredentialGenerator() {
        return credGen;
    }

    /**
     * Sets the <code>CredentialGenerator</code> object for the client
     * security for the connection.
     * 
     * @param credGen a CredentialGenerator object  
     * 
     * @see CredentialGenerator  
     */
    public void setCredentialGenerator(CredentialGenerator credGen) {
        this.credGen = credGen;
    }
    
    /**
     * Set to true to retrieve a connection with a {@link Session} that has the {@link Session#getTxCommitProtocol()}
     * set to {@link TxCommitProtocol#TWOPHASE}.
     *  
     * @param mptEnabled set to true if the Session should be configured to write to multiple partitions.
     * @since 8.6
     */
    public void setMultiPartitionSupportEnabled(boolean mptEnabled) {
    	this.multiPartitionSupportEnabled = mptEnabled;
    }
    
    /**
     * Answers true if the {@link Session} will have the {@link Session#getTxCommitProtocol()}
     * set to {@link TxCommitProtocol#TWOPHASE}.
     * @return true if the Session is capable of writing to multiple partitions.
     * @since 8.6
     */
    public boolean isMultiPartitionSupportEnabled() {
    	return this.multiPartitionSupportEnabled;
    }

    @Override
    public Object clone() {
        try {
            return super.clone(); // bitwise copy ok for all instance variables.
        }
        catch (CloneNotSupportedException e) {
            // This never happens.
            throw new Error(e);
        }
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + (beginNoWriteThrough ? 1231 : 1237);
        result = prime * result + ((credGen == null) ? 0 : credGen.hashCode());
        result = prime * result + ((objectGridName == null) ? 0 : objectGridName.hashCode());
        result = prime * result + ((og == null) ? 0 : og.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        return equalsStrict(obj, true);
    }
    
    /**
     * NOTE: This method is used via reflection externally to this
     * class because we don't want to make it part of the public API.
     * 
     * NOTE: Package protected so that doesn't show up in Javadoc.
     */
    boolean equalsStrict(Object obj, boolean strictGridInstance) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        XSConnectionSpec other = (XSConnectionSpec) obj;
        if (beginNoWriteThrough != other.beginNoWriteThrough)
            return false;
        if (credGen == null) {
            if (other.credGen != null)
                return false;
        } else if (!credGen.equals(other.credGen))
            return false;
        if (objectGridName == null) {
            if (other.objectGridName != null)
                return false;
        } else if (!objectGridName.equals(other.objectGridName)) {
            return false;
        } else if (this.multiPartitionSupportEnabled != other.multiPartitionSupportEnabled) {
        	return false;
        }
            
        // Identity comparison for local instance/shard access
        if (strictGridInstance && this.og != other.og) {
            return false;
        }
        // else { null or equivalent. Both ok }
        return true;
    }

    @Override
    public String toString() {
        StringBuilder builder = new StringBuilder();
        builder.append("XSConnectionSpec [beginNoWriteThrough=");
        builder.append(beginNoWriteThrough);
        builder.append(", ");
        builder.append("mpt="+this.multiPartitionSupportEnabled+", ");
        if (objectGridName != null) {
            builder.append("objectGridName=");
            builder.append(objectGridName);
            builder.append(", ");
        }
        if (credGen != null) {
            builder.append("credGen=");
            builder.append(credGen);
        }
        if (og != null) {
            builder.append("LocalGrid=");
            builder.append(og.getClass().getSimpleName());
            builder.append("@");
            builder.append(System.identityHashCode(og));
            builder.append(": ");
            builder.append(((ObjectGridImpl)og).getOSGiID());
        }
        builder.append("]");
        return builder.toString();
    }

 
}
