//IBM Confidential OCO Source Material
//5724-J34 (C) COPYRIGHT International Business Machines Corp. 2012
//The source code for this program is not published or otherwise divested
//of its trade secrets, irrespective of what has been deposited with the
//U.S. Copyright Office.
//
//Source File Description:
//
//    See javadoc.
//
//Change Activity:
//
//Reason       Version   Date         Userid    Change Description
//------------ --------- ------------ --------- -----------------------------------------
// wi906       8.5       02/29/2012   cdjohnson new part
//------------ --------- ------------ --------- -----------------------------------------
package com.ibm.websphere.xs.ra;

import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import com.ibm.ws.xs.ra.spi.XSRAImpl;

/**
 * The eXtreme Scale resource adapter
 * 
 * @since 8.5
 * @since XC10
 */
public final class XSResourceAdapter implements ResourceAdapter {
		
	private ResourceAdapter xsRA = null;
	
	private enum State {
        NEW, INITIALIZING, INITIALIZED, DESTROYING, DESTROYED;
    };
	
    private transient State state = State.NEW;
    
    /*
     * Instantiates the underlying impl object. 
     * Warning: do not call this directly from the constructor otherwise WAS introspection on BASE 
     * installations will fail.
     */
	private void instantiateImpl() {
		
		if (state == State.NEW) {
			synchronized (this) {
				if (state != State.NEW)
                    return;
				
				state = State.INITIALIZING;
				xsRA = new XSRAImpl();
				state = State.INITIALIZED;
			}
		}	
	}
	
	/**
	 * This method does nothing as endpoint activation is not supported
	 * 
	 * @throws ResourceException
	 * @see ResourceAdapter#endpointActivation(MessageEndpointFactory,ActivationSpec)
	 */
	public void endpointActivation(MessageEndpointFactory msgEndpointFactory,
			ActivationSpec actSpec) throws ResourceException {
		instantiateImpl();
		xsRA.endpointActivation(msgEndpointFactory, actSpec);
	}

	/**
	 * This method does nothing as endpoint deactivation is not supported
	 * 
	 * @throws ResourceException
	 * @see ResourceAdapter#endpointDeactivation(MessageEndpointFactory,ActivationSpec)
	 */
	public void endpointDeactivation(MessageEndpointFactory msgEndpointFactory,
			ActivationSpec spec) {
		instantiateImpl();
		xsRA.endpointDeactivation(msgEndpointFactory, spec);
	}

	/**
	 * This method returns null as the XA protocol is not supported
	 * 
	 * @throws ResourceException
	 * @see ResourceAdapter#getXAResources(ActivationSpec[])
	 */
	public XAResource[] getXAResources(ActivationSpec[] spec)
			throws ResourceException {
		instantiateImpl();
		return xsRA.getXAResources(spec);
	}

	/**
	 * This method initializes the eXtreme Scale resource adapter instance
	 * 
	 * @throws ResourceAdapterInternalException
	 * @see ResourceAdapter#start(BootstrapContext)
	 */
	public void start(BootstrapContext ctx)
			throws ResourceAdapterInternalException  {
		try {
			instantiateImpl();
			xsRA.start(ctx);
		} catch(NoClassDefFoundError e) {
			// Attempt was made to call the RA without the wsogclient.jar 
			// in the classpath
			throw new ResourceAdapterInternalException("Objectgrid client wsogclient.jar not found in classpath");
		}
		
	}

	/**
	 * This method performs the shutdown of the eXtreme Scale resource adapter instance
	 * @see ResourceAdapter#stop()
	 */
	public void stop() {
		if (state == State.INITIALIZED) {
			synchronized(this) {
				if (state != State.INITIALIZED) {
            		return;
            	}
				state = State.DESTROYING;
				if (xsRA != null) {
					xsRA.stop();
				}
				state = State.DESTROYED;
			}
		}
	}
	
    /**
     * Warning: This method is used via reflection. Do not change its signature without updating the reflection code.
     * * The reflection lookup is called from:
     *   - com.ibm.ws.xs.ra.spi.XSMConFactoryImpl.setResourceAdapter()
     */
	@SuppressWarnings("unused")
	private ResourceAdapter getImpl() {
	    instantiateImpl();
		return xsRA;
	}
	
    public String toString() {
        return super.toString() + "[" + (xsRA == null ? "none" : xsRA.toString() ) + "]";   
    }

}
